@@ -22,8 +22,9 @@
 
 import unittest
 
-from ldap3 import ANONYMOUS, SASL
-from test import test_sasl_user, test_sasl_password, random_id, get_connection, drop_connection, test_sasl_realm
+from ldap3 import ANONYMOUS, SASL, NTLM
+from test import test_sasl_user, test_sasl_password, random_id, get_connection, drop_connection, test_sasl_realm, test_server_type, \
+    test_ntlm_user, test_ntlm_password
 
 testcase_id = random_id()
 
@@ -50,5 +51,24 @@ class Test(unittest.TestCase):
         connection.open()
         connection.bind()
         self.assertTrue(connection.bound)
+        if test_server_type == 'EDIR':
+            connected_user = connection.extend.novell.get_bind_dn()
+            for component in test_sasl_user.split('.'):
+                self.assertTrue(component in connected_user)
+        else:
+            connected_user = str(connection.extend.standard.who_am_i())
+            self.assertEqual(connected_user, test_sasl_user)
         drop_connection(connection)
         self.assertFalse(connection.bound)
+
+    def test_ntlm(self):
+        if test_server_type == 'AD':
+            connection = get_connection(bind=False, authentication=NTLM, ntlm_credentials=(test_ntlm_user, test_ntlm_password))
+            connection.open()
+            connection.bind()
+            self.assertTrue(connection.bound)
+            connected_user = str(connection.extend.standard.who_am_i())[2:]
+            self.assertEqual(connected_user, test_ntlm_user)
+            drop_connection(connection)
+            self.assertFalse(connection.bound)
+
